@@ -46,14 +46,16 @@ import sys
 import types
 
 from _drgn import (
+    NULL,
     Architecture,
     FaultError,
     FindObjectFlags,
+    IntegerLike,
     Language,
     MissingDebugInfoError,
-    NULL,
     Object,
     OutOfBoundsError,
+    Path,
     Platform,
     PlatformFlags,
     PrimitiveType,
@@ -69,41 +71,30 @@ from _drgn import (
     TypeKind,
     TypeMember,
     TypeParameter,
-    _with_libkdumpfile,
-    array_type,
-    bool_type,
+    _with_libkdumpfile as _with_libkdumpfile,
     cast,
-    class_type,
-    complex_type,
     container_of,
-    enum_type,
     filename_matches,
-    float_type,
-    function_type,
     host_platform,
-    int_type,
-    pointer_type,
     program_from_core_dump,
     program_from_kernel,
     program_from_pid,
     reinterpret,
     sizeof,
-    struct_type,
-    typedef_type,
-    union_type,
-    void_type,
 )
-
+from drgn.internal.version import version as __version__
 
 __all__ = (
     "Architecture",
     "FaultError",
     "FindObjectFlags",
+    "IntegerLike",
     "Language",
     "MissingDebugInfoError",
     "NULL",
     "Object",
     "OutOfBoundsError",
+    "Path",
     "Platform",
     "PlatformFlags",
     "PrimitiveType",
@@ -119,37 +110,25 @@ __all__ = (
     "TypeKind",
     "TypeMember",
     "TypeParameter",
-    "array_type",
-    "bool_type",
     "cast",
-    "class_type",
-    "complex_type",
     "container_of",
-    "enum_type",
     "execscript",
     "filename_matches",
-    "float_type",
-    "function_type",
     "host_platform",
-    "int_type",
-    "pointer_type",
     "program_from_core_dump",
     "program_from_kernel",
     "program_from_pid",
     "reinterpret",
     "sizeof",
-    "struct_type",
-    "typedef_type",
-    "union_type",
-    "void_type",
 )
 
 
-try:
+if sys.version_info >= (3, 8):
     _open_code = io.open_code
-except AttributeError:
+else:
+    from typing import BinaryIO
 
-    def _open_code(path):
+    def _open_code(path: str) -> BinaryIO:
         return open(path, "rb")
 
 
@@ -176,40 +155,35 @@ def execscript(path: str, *args: str) -> None:
     added back to the calling context.
 
     This is most useful for executing scripts from interactive mode. For
-    example, you could have a script named ``tasks.py``:
+    example, you could have a script named ``exe.py``:
 
     .. code-block:: python3
 
+        \"\"\"Get all tasks executing a given file.\"\"\"
+
         import sys
 
-        \"\"\"
-        Get all tasks in a given state.
-        \"\"\"
+        from drgn.helpers.linux.fs import d_path
+        from drgn.helpers.linux.pid import find_task
 
-        # From include/linux/sched.h.
-        def task_state_index(task):
-            task_state = task.state.value_()
-            if task_state == 0x402:  # TASK_IDLE
-                return 8
+        def task_exe_path(task):
+            if task.mm:
+                return d_path(task.mm.exe_file.f_path).decode()
             else:
-                state = (task_state | task.exit_state.value_()) & 0x7f
-                return state.bit_length()
-
-        def task_state_to_char(task):
-            return 'RSDTtXZPI'[task_state_index(task)]
+                return None
 
         tasks = [
             task for task in for_each_task(prog)
-            if task_state_to_char(task) == sys.argv[1]
+            if task_exe_path(task) == sys.argv[1]
         ]
 
     Then, you could execute it and use the defined variables and functions:
 
-    >>> execscript('tasks.py', 'R')
-    >>> tasks[0].comm
-    (char [16])"python3"
-    >>> task_state_to_char(find_task(prog, 1))
-    'S'
+    >>> execscript('exe.py', '/usr/bin/bash')
+    >>> tasks[0].pid
+    (pid_t)358442
+    >>> task_exe_path(find_task(prog, 357954))
+    '/usr/bin/vim'
 
     :param path: File path of the script.
     :param args: Zero or more additional arguments to pass to the script. This
