@@ -10,7 +10,9 @@ Linux block layer, including disks (``struct gendisk``) and partitions
 (``struct hd_struct``).
 """
 
-from drgn import container_of
+from typing import Iterator
+
+from drgn import Object, Program, container_of
 from drgn.helpers import escape_ascii_string
 from drgn.helpers.linux.device import MAJOR, MINOR, MKDEV
 from drgn.helpers.linux.list import list_for_each_entry
@@ -27,34 +29,33 @@ __all__ = (
 )
 
 
-def disk_devt(disk):
+def disk_devt(disk: Object) -> Object:
     """
-    .. c:function:: dev_t disk_devt(struct gendisk *disk)
-
     Get a disk's device number.
+
+    :param disk: ``struct gendisk *``
+    :return: ``dev_t``
     """
-    return MKDEV(disk.major, disk.first_minor)
+    return Object(disk.prog_, "dev_t", MKDEV(disk.major, disk.first_minor))
 
 
-def disk_name(disk):
+def disk_name(disk: Object) -> bytes:
     """
-    .. c:function:: char *disk_name(struct gendisk *disk)
-
     Get the name of a disk (e.g., ``sda``).
 
-    :rtype: bytes
+    :param disk: ``struct gendisk *``
     """
     return disk.disk_name.string_()
 
 
-def _for_each_block_device(prog):
+def _for_each_block_device(prog: Program) -> Iterator[Object]:
     try:
         class_in_private = prog.cache["knode_class_in_device_private"]
     except KeyError:
         # We need a proper has_member(), but this is fine for now.
         class_in_private = any(
             member.name == "knode_class"
-            for member in prog.type("struct device_private").members
+            for member in prog.type("struct device_private").members  # type: ignore[union-attr]
         )
         prog.cache["knode_class_in_device_private"] = class_in_private
     devices = prog["block_class"].p.klist_devices.k_list.address_of_()
@@ -67,7 +68,7 @@ def _for_each_block_device(prog):
         yield from list_for_each_entry("struct device", devices, "knode_class.n_node")
 
 
-def for_each_disk(prog):
+def for_each_disk(prog: Program) -> Iterator[Object]:
     """
     Iterate over all disks in the system.
 
@@ -79,7 +80,7 @@ def for_each_disk(prog):
             yield container_of(device, "struct gendisk", "part0.__dev")
 
 
-def print_disks(prog):
+def print_disks(prog: Program) -> None:
     """Print all of the disks in the system."""
     for disk in for_each_disk(prog):
         major = disk.major.value_()
@@ -88,27 +89,26 @@ def print_disks(prog):
         print(f"{major}:{minor} {name} ({disk.type_.type_name()})0x{disk.value_():x}")
 
 
-def part_devt(part):
+def part_devt(part: Object) -> Object:
     """
-    .. c:function:: dev_t part_devt(struct hd_struct *part)
-
     Get a partition's device number.
+
+    :param part: ``struct hd_struct *``
+    :return: ``dev_t``
     """
     return part.__dev.devt
 
 
-def part_name(part):
+def part_name(part: Object) -> bytes:
     """
-    .. c:function:: char *part_name(struct hd_struct *part)
-
     Get the name of a partition (e.g., ``sda1``).
 
-    :rtype: bytes
+    :param part: ``struct hd_struct *``
     """
     return part.__dev.kobj.name.string_()
 
 
-def for_each_partition(prog):
+def for_each_partition(prog: Program) -> Iterator[Object]:
     """
     Iterate over all partitions in the system.
 
@@ -118,7 +118,7 @@ def for_each_partition(prog):
         yield container_of(device, "struct hd_struct", "__dev")
 
 
-def print_partitions(prog):
+def print_partitions(prog: Program) -> None:
     """Print all of the partitions in the system."""
     for part in for_each_partition(prog):
         devt = part_devt(part).value_()
