@@ -11,9 +11,11 @@ an ID to a pointer. This currently only supports Linux v4.11+; before this,
 IDRs were not based on radix trees.
 """
 
-from drgn.helpers.linux.radixtree import radix_tree_for_each, radix_tree_lookup
-from _drgn import _linux_helper_idr_find
+from typing import Iterator, Tuple
 
+from _drgn import _linux_helper_idr_find as idr_find
+from drgn import Object
+from drgn.helpers.linux.radixtree import radix_tree_for_each, radix_tree_lookup
 
 __all__ = (
     "idr_find",
@@ -21,24 +23,12 @@ __all__ = (
 )
 
 
-def idr_find(idr, id):
+def idr_for_each(idr: Object) -> Iterator[Tuple[int, Object]]:
     """
-    .. c:function:: void *idr_find(struct idr *idr, unsigned long id)
-
-    Look up the entry with the given id in an IDR. If it is not found, this
-    returns a ``NULL`` object.
-    """
-    return _linux_helper_idr_find(idr, id)
-
-
-def idr_for_each(idr):
-    """
-    .. c:function:: idr_for_each(struct idr *idr)
-
     Iterate over all of the entries in an IDR.
 
+    :param idr: ``struct idr *``
     :return: Iterator of (index, ``void *``) tuples.
-    :rtype: Iterator[tuple[int, Object]]
     """
     try:
         base = idr.idr_base.value_()
