@@ -10,31 +10,32 @@ per-CPU allocations from :linux:`include/linux/percpu.h` and per-CPU counters
 from :linux:`include/linux/percpu_counter.h`.
 """
 
-from drgn import Object
+from drgn import IntegerLike, Object
 from drgn.helpers.linux.cpumask import for_each_online_cpu
 
-
 __all__ = (
     "per_cpu_ptr",
     "percpu_counter_sum",
 )
 
 
-def per_cpu_ptr(ptr, cpu):
+def per_cpu_ptr(ptr: Object, cpu: IntegerLike) -> Object:
     """
-    .. c:function:: type *per_cpu_ptr(type __percpu *ptr, int cpu)
-
     Return the per-CPU pointer for a given CPU.
+
+    :param ptr: ``type __percpu *``
+    :param cpu: CPU number.
+    :return: ``type *``
     """
     offset = ptr.prog_["__per_cpu_offset"][cpu].value_()
     return Object(ptr.prog_, ptr.type_, value=ptr.value_() + offset)
 
 
-def percpu_counter_sum(fbc):
+def percpu_counter_sum(fbc: Object) -> int:
     """
-    .. c:function:: s64 percpu_counter_sum(struct percpu_counter *fbc)
-
     Return the sum of a per-CPU counter.
+
+    :param fbc: ``struct percpu_counter *``
     """
     ret = fbc.count.value_()
     ptr = fbc.counters
