@@ -9,8 +9,9 @@ The ``drgn.helpers.linux.rbtree`` module provides helpers for working with
 red-black trees from :linux:`include/linux/rbtree.h`.
 """
 
-from drgn import Object, NULL, container_of
+from typing import Callable, Iterator, TypeVar
 
+from drgn import NULL, Object, container_of
 
 __all__ = (
     "RB_EMPTY_NODE",
@@ -25,31 +26,33 @@ __all__ = (
 )
 
 
-def RB_EMPTY_NODE(node):
+def RB_EMPTY_NODE(node: Object) -> bool:
     """
-    .. c:function:: bool RB_EMPTY_NODE(struct rb_node *node)
-
     Return whether a red-black tree node is empty, i.e., not inserted in a
     tree.
+
+    :param node: ``struct rb_node *``
     """
     return node.__rb_parent_color.value_() == node.value_()
 
 
-def rb_parent(node):
+def rb_parent(node: Object) -> Object:
     """
-    .. c:function:: struct rb_node *rb_parent(struct rb_node *node)
-
     Return the parent node of a red-black tree node.
+
+    :param node: ``struct rb_node *``
+    :return: ``struct rb_node *``
     """
     return Object(node.prog_, node.type_, value=node.__rb_parent_color.value_() & ~3)
 
 
-def rb_first(root):
+def rb_first(root: Object) -> Object:
     """
-    .. c:function:: struct rb_node *rb_first(struct rb_root *root)
+    Return the first node (in sort order) in a red-black tree, or ``NULL`` if
+    the tree is empty.
 
-    Return the first node (in sort order) in a red-black tree, or a ``NULL``
-    object if the tree is empty.
+    :param root: ``struct rb_root *``
+    :return: ``struct rb_node *``
     """
     node = root.rb_node.read_()
     if not node:
@@ -61,12 +64,13 @@ def rb_first(root):
         node = next
 
 
-def rb_last(root):
+def rb_last(root: Object) -> Object:
     """
-    .. c:function:: struct rb_node *rb_last(struct rb_root *root)
+    Return the last node (in sort order) in a red-black tree, or ``NULL`` if
+    the tree is empty.
 
-    Return the last node (in sort order) in a red-black tree, or a ``NULL``
-    object if the tree is empty.
+    :param root: ``struct rb_root *``
+    :return: ``struct rb_node *``
     """
     node = root.rb_node.read_()
     if not node:
@@ -78,12 +82,13 @@ def rb_last(root):
         node = next
 
 
-def rb_next(node):
+def rb_next(node: Object) -> Object:
     """
-    .. c:function:: struct rb_node *rb_next(struct rb_node *node)
+    Return the next node (in sort order) after a red-black node, or ``NULL`` if
+    the node is the last node in the tree or is empty.
 
-    Return the next node (in sort order) after a red-black node, or a ``NULL``
-    object if the node is the last node in the tree or is empty.
+    :param node: ``struct rb_node *``
+    :return: ``struct rb_node *``
     """
     node = node.read_()
 
@@ -106,12 +111,13 @@ def rb_next(node):
     return parent
 
 
-def rb_prev(node):
+def rb_prev(node: Object) -> Object:
     """
-    .. c:function:: struct rb_node *rb_prev(struct rb_node *node)
+    Return the previous node (in sort order) before a red-black node, or
+    ``NULL`` if the node is the first node in the tree or is empty.
 
-    Return the previous node (in sort order) before a red-black node, or a
-    ``NULL`` object if the node is the first node in the tree or is empty.
+    :param node: ``struct rb_node *``
+    :return: ``struct rb_node *``
     """
     node = node.read_()
 
@@ -134,16 +140,15 @@ def rb_prev(node):
     return parent
 
 
-def rbtree_inorder_for_each(root):
+def rbtree_inorder_for_each(root: Object) -> Iterator[Object]:
     """
-    .. c:function:: rbtree_inorder_for_each(struct rb_root *root)
-
     Iterate over all of the nodes in a red-black tree, in sort order.
 
+    :param root: ``struct rb_root *``
     :return: Iterator of ``struct rb_node *`` objects.
     """
 
-    def aux(node):
+    def aux(node: Object) -> Iterator[Object]:
         if node:
             yield from aux(node.rb_left.read_())
             yield node
@@ -152,32 +157,45 @@ def rbtree_inorder_for_each(root):
     yield from aux(root.rb_node.read_())
 
 
-def rbtree_inorder_for_each_entry(type, root, member):
+def rbtree_inorder_for_each_entry(
+    type: str, root: Object, member: str
+) -> Iterator[Object]:
     """
-    .. c:function:: rbtree_inorder_for_each_entry(type, struct rb_root *root, member)
-
-    Iterate over all of the entries in a red-black tree, given the type of the
-    entry and the ``struct rb_node`` member in that type. The entries are
-    returned in sort order.
+    Iterate over all of the entries in a red-black tree in sorted order.
 
+    :param type: Entry type.
+    :param root: ``struct rb_root *``
+    :param member: Name of red-black node member in entry type.
     :return: Iterator of ``type *`` objects.
     """
     for node in rbtree_inorder_for_each(root):
         yield container_of(node, type, member)
 
 
-def rb_find(type, root, member, key, cmp):
-    """
-    .. c:function:: type *rb_find(type, struct rb_root *root, member, key_type key, int (*cmp)(key_type, type *))
+KeyType = TypeVar("KeyType")
 
-    Find an entry in a red-black tree, given a key and a comparator function
-    which takes the key and an entry. The comparator should return < 0 if the
-    key is less than the entry, > 0 if it is greater than the entry, or 0 if it
-    matches the entry. This returns a ``NULL`` object if no entry matches the
-    key.
+
+def rb_find(
+    type: str,
+    root: Object,
+    member: str,
+    key: KeyType,
+    cmp: Callable[[KeyType, Object], int],
+) -> Object:
+    """
+    Find an entry in a red-black tree given a key and a comparator function.
 
     Note that this function does not have an analogue in the Linux kernel
     source code, as tree searches are all open-coded.
+
+    :param type: Entry type.
+    :param root: ``struct rb_root *``
+    :param member: Name of red-black node member in entry type.
+    :param key: Key to find.
+    :param cmp: Callback taking key and entry that returns < 0 if the key is
+        less than the entry, > 0 if the key is greater than the entry, and 0 if
+        the key matches the entry.
+    :return: ``type *`` found entry, or ``NULL`` if not found.
     """
     node = root.rb_node.read_()
     while node:
@@ -189,4 +207,4 @@ def rb_find(type, root, member, key, cmp):
             node = node.rb_right.read_()
         else:
             return entry
-    return node
+    return NULL(root.prog_, type)
