@@ -9,15 +9,16 @@ The ``drgn.helpers.linux.tcp`` module provides helpers for working with the TCP
 protocol in the Linux kernel.
 """
 
-from drgn import cast
+from drgn import Object, cast
 
 __all__ = ("sk_tcpstate",)
 
 
-def sk_tcpstate(sk):
+def sk_tcpstate(sk: Object) -> Object:
     """
-    .. c:function:: enum TcpState sk_tcpstate(struct sock *sk)
-
     Return the TCP protocol state of a socket.
+
+    :param sk: ``struct sock *``
+    :return: TCP state enum value.
     """
     return cast(sk.prog_["TCP_ESTABLISHED"].type_, sk.__sk_common.skc_state)
