@@ -1,24 +1,29 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
+#include <assert.h>
 #include <ctype.h>
 #include <float.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <math.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include "internal.h"
+#include "bitops.h"
+#include "error.h"
 #include "hash_table.h"
-#include "language.h"
+#include "language.h" // IWYU pragma: associated
 #include "lexer.h"
 #include "memory_reader.h"
+#include "minmax.h"
 #include "object.h"
 #include "program.h"
 #include "string_builder.h"
 #include "symbol.h"
 #include "type.h"
-#include "type_index.h"
+#include "util.h"
+#include "vector.h"
 
 static struct drgn_error *
 c_declare_variable(struct drgn_qualified_type qualified_type,
@@ -1160,7 +1165,7 @@ c_format_compound_object(const struct drgn_object *obj,
 	    new->member < new->end) {
 		struct drgn_object member;
 
-		drgn_object_init(&member, obj->prog);
+		drgn_object_init(&member, drgn_object_program(obj));
 		do {
 			struct drgn_qualified_type member_type;
 			bool zero;
@@ -1188,7 +1193,7 @@ c_format_compound_object(const struct drgn_object *obj,
 			return err;
 	}
 
-	err = c_format_initializer(obj->prog, &iter.iter, indent,
+	err = c_format_initializer(drgn_object_program(obj), &iter.iter, indent,
 				   one_line_columns, multi_line_columns,
 				   flags & DRGN_FORMAT_OBJECT_MEMBERS_SAME_LINE,
 				   sb);
@@ -1292,7 +1297,7 @@ c_format_pointer_object(const struct drgn_object *obj,
 		return err;
 
 	have_symbol = ((flags & DRGN_FORMAT_OBJECT_SYMBOLIZE) &&
-		       drgn_program_find_symbol_by_address_internal(obj->prog,
+		       drgn_program_find_symbol_by_address_internal(drgn_object_program(obj),
 								    uvalue,
 								    NULL,
 								    &sym));
@@ -1317,12 +1322,12 @@ c_format_pointer_object(const struct drgn_object *obj,
 		return &drgn_enomem;
 
 	if (c_string) {
-		err = c_format_string(&obj->prog->reader, uvalue, UINT64_MAX,
-				      sb);
+		err = c_format_string(&drgn_object_program(obj)->reader, uvalue,
+				      UINT64_MAX, sb);
 	} else {
 		struct drgn_object dereferenced;
 
-		drgn_object_init(&dereferenced, obj->prog);
+		drgn_object_init(&dereferenced, drgn_object_program(obj));
 		err = drgn_object_dereference(&dereferenced, obj);
 		if (err) {
 			drgn_object_deinit(&dereferenced);
@@ -1452,7 +1457,7 @@ c_format_array_object(const struct drgn_object *obj,
 	if ((flags & DRGN_FORMAT_OBJECT_STRING) && iter.length &&
 	    is_character_type(iter.element_type.type)) {
 		if (obj->is_reference) {
-			return c_format_string(&obj->prog->reader,
+			return c_format_string(&drgn_object_program(obj)->reader,
 					       obj->reference.address,
 					       iter.length, sb);
 		} else {
@@ -1492,7 +1497,7 @@ c_format_array_object(const struct drgn_object *obj,
 	    iter.length) {
 		struct drgn_object element;
 
-		drgn_object_init(&element, obj->prog);
+		drgn_object_init(&element, drgn_object_program(obj));
 		do {
 			bool zero;
 
@@ -1516,8 +1521,9 @@ c_format_array_object(const struct drgn_object *obj,
 		if (err)
 			return err;
 	}
-	return c_format_initializer(obj->prog, &iter.iter, indent,
-				    one_line_columns, multi_line_columns,
+	return c_format_initializer(drgn_object_program(obj), &iter.iter,
+				    indent, one_line_columns,
+				    multi_line_columns,
 				    flags & DRGN_FORMAT_OBJECT_ELEMENTS_SAME_LINE,
 				    sb);
 }
@@ -1680,7 +1686,7 @@ static const char *token_spelling[] = {
 	[C_TOKEN_ENUM] = "enum",
 };
 
-DEFINE_HASH_MAP(c_keyword_map, struct string, int, string_hash, string_eq);
+DEFINE_HASH_MAP(c_keyword_map, struct string, int, string_hash_pair, string_eq)
 
 static struct c_keyword_map c_keywords = HASH_TABLE_INIT;
 
@@ -2111,7 +2117,7 @@ out:
 }
 
 static struct drgn_error *
-c_parse_specifier_qualifier_list(struct drgn_type_index *tindex,
+c_parse_specifier_qualifier_list(struct drgn_program *prog,
 				 struct drgn_lexer *lexer, const char *filename,
 				 struct drgn_qualified_type *ret)
 {
@@ -2206,17 +2212,17 @@ c_parse_specifier_qualifier_list(struct drgn_type_index *tindex,
 			kind = DRGN_TYPE_ENUM;
 		} else if (identifier) {
 			if (strstartswith(identifier, "size_t")) {
-				err = drgn_type_index_find_primitive(tindex,
-								     DRGN_C_TYPE_SIZE_T,
-								     &ret->type);
+				err = drgn_program_find_primitive_type(prog,
+								       DRGN_C_TYPE_SIZE_T,
+								       &ret->type);
 				if (err)
 					return err;
 				ret->qualifiers = 0;
 				goto out;
 			} else if (strstartswith(identifier, "ptrdiff_t")) {
-				err = drgn_type_index_find_primitive(tindex,
-								     DRGN_C_TYPE_PTRDIFF_T,
-								     &ret->type);
+				err = drgn_program_find_primitive_type(prog,
+								       DRGN_C_TYPE_PTRDIFF_T,
+								       &ret->type);
 				if (err)
 					return err;
 				ret->qualifiers = 0;
@@ -2229,15 +2235,15 @@ c_parse_specifier_qualifier_list(struct drgn_type_index *tindex,
 						 "expected type specifier");
 		}
 
-		err = drgn_type_index_find_parsed(tindex, kind, identifier,
+		err = drgn_program_find_type_impl(prog, kind, identifier,
 						  identifier_len, filename,
 						  ret);
 		if (err)
 			return err;
 	} else {
-		err = drgn_type_index_find_primitive(tindex,
-						     specifier_kind[specifier],
-						     &ret->type);
+		err = drgn_program_find_primitive_type(prog,
+						       specifier_kind[specifier],
+						       &ret->type);
 		if (err)
 			return err;
 		ret->qualifiers = 0;
@@ -2259,7 +2265,7 @@ struct c_declarator {
 
 /* These functions don't free the declarator list on error. */
 static struct drgn_error *
-c_parse_abstract_declarator(struct drgn_type_index *tindex,
+c_parse_abstract_declarator(struct drgn_program *prog,
 			    struct drgn_lexer *lexer,
 			    struct c_declarator **outer,
 			    struct c_declarator **inner);
@@ -2289,7 +2295,7 @@ c_parse_optional_type_qualifier_list(struct drgn_lexer *lexer,
 }
 
 static struct drgn_error *
-c_parse_pointer(struct drgn_type_index *tindex, struct drgn_lexer *lexer,
+c_parse_pointer(struct drgn_program *prog, struct drgn_lexer *lexer,
 		struct c_declarator **outer, struct c_declarator **inner)
 {
 	struct drgn_error *err;
@@ -2329,7 +2335,7 @@ c_parse_pointer(struct drgn_type_index *tindex, struct drgn_lexer *lexer,
 }
 
 static struct drgn_error *
-c_parse_direct_abstract_declarator(struct drgn_type_index *tindex,
+c_parse_direct_abstract_declarator(struct drgn_program *prog,
 				   struct drgn_lexer *lexer,
 				   struct c_declarator **outer,
 				   struct c_declarator **inner)
@@ -2351,7 +2357,7 @@ c_parse_direct_abstract_declarator(struct drgn_type_index *tindex,
 		if (token2.kind == C_TOKEN_ASTERISK ||
 		    token2.kind == C_TOKEN_LPAREN ||
 		    token2.kind == C_TOKEN_LBRACKET) {
-			err = c_parse_abstract_declarator(tindex, lexer, outer,
+			err = c_parse_abstract_declarator(prog, lexer, outer,
 							  inner);
 			if (err)
 				return err;
@@ -2431,7 +2437,7 @@ c_parse_direct_abstract_declarator(struct drgn_type_index *tindex,
 }
 
 static struct drgn_error *
-c_parse_abstract_declarator(struct drgn_type_index *tindex,
+c_parse_abstract_declarator(struct drgn_program *prog,
 			    struct drgn_lexer *lexer,
 			    struct c_declarator **outer,
 			    struct c_declarator **inner)
@@ -2443,7 +2449,7 @@ c_parse_abstract_declarator(struct drgn_type_index *tindex,
 	if (err)
 		return err;
 	if (token.kind == C_TOKEN_ASTERISK) {
-		err = c_parse_pointer(tindex, lexer, outer, inner);
+		err = c_parse_pointer(prog, lexer, outer, inner);
 		if (err)
 			return err;
 
@@ -2452,21 +2458,21 @@ c_parse_abstract_declarator(struct drgn_type_index *tindex,
 		    token.kind == C_TOKEN_LBRACKET) {
 			struct c_declarator *tmp;
 
-			err = c_parse_direct_abstract_declarator(tindex, lexer,
+			err = c_parse_direct_abstract_declarator(prog, lexer,
 								 outer, &tmp);
 			if (err)
 				return err;
 		}
 		return NULL;
 	} else {
-		return c_parse_direct_abstract_declarator(tindex, lexer, outer,
+		return c_parse_direct_abstract_declarator(prog, lexer, outer,
 							  inner);
 	}
 }
 
 /* This always frees the declarator list regardless of success or failure. */
 static struct drgn_error *
-c_type_from_declarator(struct drgn_type_index *tindex,
+c_type_from_declarator(struct drgn_program *prog,
 		       struct c_declarator *declarator,
 		       struct drgn_qualified_type *ret)
 {
@@ -2475,21 +2481,28 @@ c_type_from_declarator(struct drgn_type_index *tindex,
 	if (!declarator)
 		return NULL;
 
-	err = c_type_from_declarator(tindex, declarator->next, ret);
+	err = c_type_from_declarator(prog, declarator->next, ret);
 	if (err) {
 		free(declarator);
 		return err;
 	}
 
 	if (declarator->kind == C_TOKEN_ASTERISK) {
-		err = drgn_type_index_pointer_type(tindex, *ret, NULL,
-						   &ret->type);
+		uint8_t word_size;
+		err = drgn_program_word_size(prog, &word_size);
+		if (!err) {
+			err = drgn_pointer_type_create(prog, *ret, word_size,
+						       drgn_type_language(ret->type),
+						       &ret->type);
+		}
 	} else if (declarator->is_complete) {
-		err = drgn_type_index_array_type(tindex, declarator->length,
-						 *ret, NULL, &ret->type);
+		err = drgn_array_type_create(prog, *ret, declarator->length,
+					     drgn_type_language(ret->type),
+					     &ret->type);
 	} else {
-		err = drgn_type_index_incomplete_array_type(tindex, *ret, NULL,
-							    &ret->type);
+		err = drgn_incomplete_array_type_create(prog, *ret,
+							drgn_type_language(ret->type),
+							&ret->type);
 	}
 
 	if (!err)
@@ -2498,7 +2511,7 @@ c_type_from_declarator(struct drgn_type_index *tindex,
 	return err;
 }
 
-struct drgn_error *c_find_type(struct drgn_type_index *tindex, const char *name,
+struct drgn_error *c_find_type(struct drgn_program *prog, const char *name,
 			       const char *filename,
 			       struct drgn_qualified_type *ret)
 {
@@ -2508,7 +2521,7 @@ struct drgn_error *c_find_type(struct drgn_type_index *tindex, const char *name,
 
 	drgn_lexer_init(&lexer, drgn_lexer_c, name);
 
-	err = c_parse_specifier_qualifier_list(tindex, &lexer, filename, ret);
+	err = c_parse_specifier_qualifier_list(prog, &lexer, filename, ret);
 	if (err)
 		goto out;
 
@@ -2522,8 +2535,7 @@ struct drgn_error *c_find_type(struct drgn_type_index *tindex, const char *name,
 		if (err)
 			return err;
 
-		err = c_parse_abstract_declarator(tindex, &lexer, &outer,
-						  &inner);
+		err = c_parse_abstract_declarator(prog, &lexer, &outer, &inner);
 		if (err) {
 			while (outer) {
 				struct c_declarator *next;
@@ -2535,7 +2547,7 @@ struct drgn_error *c_find_type(struct drgn_type_index *tindex, const char *name,
 			goto out;
 		}
 
-		err = c_type_from_declarator(tindex, outer, ret);
+		err = c_type_from_declarator(prog, outer, ret);
 		if (err)
 			goto out;
 
@@ -2580,11 +2592,10 @@ struct drgn_error *c_bit_offset(struct drgn_program *prog,
 				struct drgn_member_value *member;
 				struct drgn_qualified_type member_type;
 
-				err = drgn_type_index_find_member(&prog->tindex,
-								  type,
-								  token.value,
-								  token.len,
-								  &member);
+				err = drgn_program_find_member(prog, type,
+							       token.value,
+							       token.len,
+							       &member);
 				if (err)
 					goto out;
 				if (__builtin_add_overflow(bit_offset,
@@ -2703,9 +2714,9 @@ struct drgn_error *c_integer_literal(struct drgn_object *res, uint64_t uvalue)
 	bits = fls(uvalue);
 	qualified_type.qualifiers = 0;
 	for (i = 0; i < ARRAY_SIZE(types); i++) {
-		err = drgn_type_index_find_primitive(&res->prog->tindex,
-						     types[i],
-						     &qualified_type.type);
+		err = drgn_program_find_primitive_type(drgn_object_program(res),
+						       types[i],
+						       &qualified_type.type);
 		if (err)
 			return err;
 
@@ -2729,9 +2740,9 @@ struct drgn_error *c_bool_literal(struct drgn_object *res, bool bvalue)
 	struct drgn_error *err;
 	struct drgn_qualified_type qualified_type;
 
-	err = drgn_type_index_find_primitive(&res->prog->tindex,
-					     DRGN_C_TYPE_INT,
-					     &qualified_type.type);
+	err = drgn_program_find_primitive_type(drgn_object_program(res),
+					       DRGN_C_TYPE_INT,
+					       &qualified_type.type);
 	if (err)
 		return err;
 	qualified_type.qualifiers = 0;
@@ -2743,9 +2754,9 @@ struct drgn_error *c_float_literal(struct drgn_object *res, double fvalue)
 	struct drgn_error *err;
 	struct drgn_qualified_type qualified_type;
 
-	err = drgn_type_index_find_primitive(&res->prog->tindex,
-					     DRGN_C_TYPE_DOUBLE,
-					     &qualified_type.type);
+	err = drgn_program_find_primitive_type(drgn_object_program(res),
+					       DRGN_C_TYPE_DOUBLE,
+					       &qualified_type.type);
 	if (err)
 		return err;
 	qualified_type.qualifiers = 0;
@@ -2800,7 +2811,7 @@ static bool c_can_represent_all_values(struct drgn_type *type1,
 		return false;
 }
 
-static struct drgn_error *c_integer_promotions(struct drgn_type_index *tindex,
+static struct drgn_error *c_integer_promotions(struct drgn_program *prog,
 					       struct drgn_object_type *type)
 {
 	struct drgn_error *err;
@@ -2850,8 +2861,8 @@ static struct drgn_error *c_integer_promotions(struct drgn_type_index *tindex,
 	 */
 	if (primitive >= ARRAY_SIZE(c_integer_conversion_rank) ||
 	    type->bit_field_size) {
-		err = drgn_type_index_find_primitive(tindex, DRGN_C_TYPE_INT,
-						     &int_type);
+		err = drgn_program_find_primitive_type(prog, DRGN_C_TYPE_INT,
+						       &int_type);
 		if (err)
 			return err;
 		if (c_can_represent_all_values(int_type, 0,
@@ -2862,9 +2873,9 @@ static struct drgn_error *c_integer_promotions(struct drgn_type_index *tindex,
 			return NULL;
 		}
 
-		err = drgn_type_index_find_primitive(tindex,
-						     DRGN_C_TYPE_UNSIGNED_INT,
-						     &int_type);
+		err = drgn_program_find_primitive_type(prog,
+						       DRGN_C_TYPE_UNSIGNED_INT,
+						       &int_type);
 		if (err)
 			return err;
 		if (c_can_represent_all_values(int_type, 0,
@@ -2886,16 +2897,16 @@ static struct drgn_error *c_integer_promotions(struct drgn_type_index *tindex,
 	 * If int can represent all values of the original type, then the result
 	 * is int. Otherwise, the result is unsigned int.
 	 */
-	err = drgn_type_index_find_primitive(tindex, DRGN_C_TYPE_INT,
-					     &int_type);
+	err = drgn_program_find_primitive_type(prog, DRGN_C_TYPE_INT,
+					       &int_type);
 	if (err)
 		return err;
 	if (c_can_represent_all_values(int_type, 0, type->underlying_type, 0)) {
 		type->type = int_type;
 	} else {
-		err = drgn_type_index_find_primitive(tindex,
-						     DRGN_C_TYPE_UNSIGNED_INT,
-						     &type->type);
+		err = drgn_program_find_primitive_type(prog,
+						       DRGN_C_TYPE_UNSIGNED_INT,
+						       &type->type);
 		if (err)
 			return err;
 	}
@@ -2904,7 +2915,7 @@ static struct drgn_error *c_integer_promotions(struct drgn_type_index *tindex,
 }
 
 static struct drgn_error *
-c_corresponding_unsigned_type(struct drgn_type_index *tindex,
+c_corresponding_unsigned_type(struct drgn_program *prog,
 			      enum drgn_primitive_type type,
 			      struct drgn_type **ret)
 {
@@ -2914,23 +2925,23 @@ c_corresponding_unsigned_type(struct drgn_type_index *tindex,
 	 * handle them here.
 	 */
 	case DRGN_C_TYPE_INT:
-		return drgn_type_index_find_primitive(tindex,
-						      DRGN_C_TYPE_UNSIGNED_INT,
-						      ret);
+		return drgn_program_find_primitive_type(prog,
+							DRGN_C_TYPE_UNSIGNED_INT,
+							ret);
 	case DRGN_C_TYPE_LONG:
-		return drgn_type_index_find_primitive(tindex,
-						      DRGN_C_TYPE_UNSIGNED_LONG,
-						      ret);
+		return drgn_program_find_primitive_type(prog,
+							DRGN_C_TYPE_UNSIGNED_LONG,
+							ret);
 	case DRGN_C_TYPE_LONG_LONG:
-		return drgn_type_index_find_primitive(tindex,
-						      DRGN_C_TYPE_UNSIGNED_LONG_LONG,
-						      ret);
+		return drgn_program_find_primitive_type(prog,
+							DRGN_C_TYPE_UNSIGNED_LONG_LONG,
+							ret);
 	default:
 		UNREACHABLE();
 	}
 }
 
-static struct drgn_error *c_common_real_type(struct drgn_type_index *tindex,
+static struct drgn_error *c_common_real_type(struct drgn_program *prog,
 					     struct drgn_object_type *type1,
 					     struct drgn_object_type *type2,
 					     struct drgn_object_type *ret)
@@ -2980,10 +2991,10 @@ static struct drgn_error *c_common_real_type(struct drgn_type_index *tindex,
 	 * Otherwise, the integer promotions are performed before applying the
 	 * following rules.
 	 */
-	err = c_integer_promotions(tindex, type1);
+	err = c_integer_promotions(prog, type1);
 	if (err)
 		return err;
-	err = c_integer_promotions(tindex, type2);
+	err = c_integer_promotions(prog, type2);
 	if (err)
 		return err;
 
@@ -3103,7 +3114,7 @@ static struct drgn_error *c_common_real_type(struct drgn_type_index *tindex,
 	 * rank, then it must have greater size and thus be able to represent
 	 * all values of the unsigned integer type.
 	 */
-	err = c_corresponding_unsigned_type(tindex,
+	err = c_corresponding_unsigned_type(prog,
 					    is_signed1 ? primitive1 : primitive2,
 					    &ret->type);
 	if (err)
@@ -3129,25 +3140,36 @@ static struct drgn_error *c_operand_type(const struct drgn_object *obj,
 
 	*type_ret = drgn_object_type(obj);
 	switch (drgn_type_kind(type_ret->underlying_type)) {
-	case DRGN_TYPE_ARRAY:
-		err = drgn_type_index_pointer_type(&obj->prog->tindex,
-						   drgn_type_type(type_ret->underlying_type),
-						   drgn_type_language(type_ret->underlying_type),
-						   &type_ret->type);
+	case DRGN_TYPE_ARRAY: {
+		uint8_t word_size;
+		err = drgn_program_word_size(drgn_object_program(obj),
+					     &word_size);
+		if (err)
+			return err;
+		err = drgn_pointer_type_create(drgn_object_program(obj),
+					       drgn_type_type(type_ret->underlying_type),
+					       word_size,
+					       drgn_type_language(type_ret->underlying_type),
+					       &type_ret->type);
 		if (err)
 			return err;
 		type_ret->underlying_type = type_ret->type;
 		break;
+	}
 	case DRGN_TYPE_FUNCTION: {
 		struct drgn_qualified_type function_type = {
 			.type = type_ret->underlying_type,
 			.qualifiers = type_ret->qualifiers,
 		};
-
-		err = drgn_type_index_pointer_type(&obj->prog->tindex,
-						   function_type,
-						   drgn_type_language(type_ret->underlying_type),
-						   &type_ret->type);
+		uint8_t word_size;
+		err = drgn_program_word_size(drgn_object_program(obj),
+					     &word_size);
+		if (err)
+			return err;
+		err = drgn_pointer_type_create(drgn_object_program(obj),
+					       function_type, word_size,
+					       drgn_type_language(type_ret->underlying_type),
+					       &type_ret->type);
 		if (err)
 			return err;
 		type_ret->underlying_type = type_ret->type;
@@ -3257,7 +3279,7 @@ struct drgn_error *c_op_cmp(const struct drgn_object *lhs,
 		if (!drgn_type_is_arithmetic(lhs_type.underlying_type) ||
 		    !drgn_type_is_arithmetic(rhs_type.underlying_type))
 			goto type_error;
-		err = c_common_real_type(&lhs->prog->tindex, &lhs_type,
+		err = c_common_real_type(drgn_object_program(lhs), &lhs_type,
 					 &rhs_type, &type);
 		if (err)
 			return err;
@@ -3299,7 +3321,7 @@ struct drgn_error *c_op_add(struct drgn_object *res,
 		if (!drgn_type_is_arithmetic(lhs_type.underlying_type) ||
 		    !drgn_type_is_arithmetic(rhs_type.underlying_type))
 			goto type_error;
-		err = c_common_real_type(&lhs->prog->tindex, &lhs_type,
+		err = c_common_real_type(drgn_object_program(lhs), &lhs_type,
 					 &rhs_type, &type);
 		if (err)
 			return err;
@@ -3330,9 +3352,9 @@ struct drgn_error *c_op_sub(struct drgn_object *res,
 	if (lhs_pointer && rhs_pointer) {
 		struct drgn_object_type type = {};
 
-		err = drgn_type_index_find_primitive(&lhs->prog->tindex,
-						     DRGN_C_TYPE_PTRDIFF_T,
-						     &type.type);
+		err = drgn_program_find_primitive_type(drgn_object_program(lhs),
+						       DRGN_C_TYPE_PTRDIFF_T,
+						       &type.type);
 		if (err)
 			return err;
 		type.underlying_type = drgn_underlying_type(type.type);
@@ -3351,7 +3373,7 @@ struct drgn_error *c_op_sub(struct drgn_object *res,
 		if (!drgn_type_is_arithmetic(lhs_type.underlying_type) ||
 		    !drgn_type_is_arithmetic(rhs_type.underlying_type))
 			goto type_error;
-		err = c_common_real_type(&lhs->prog->tindex, &lhs_type,
+		err = c_common_real_type(drgn_object_program(lhs), &lhs_type,
 					 &rhs_type, &type);
 		if (err)
 			return err;
@@ -3382,8 +3404,8 @@ struct drgn_error *c_op_##op_name(struct drgn_object *res,			\
 		return drgn_error_binary_op("binary "#op, &lhs_type,		\
 					    &rhs_type);				\
 										\
-	err = c_common_real_type(&lhs->prog->tindex, &lhs_type, &rhs_type,	\
-				 &type);					\
+	err = c_common_real_type(drgn_object_program(lhs), &lhs_type,		\
+				 &rhs_type, &type);				\
 	if (err)								\
 		return err;							\
 										\
@@ -3416,10 +3438,10 @@ struct drgn_error *c_op_##op_name(struct drgn_object *res,			\
 		return drgn_error_binary_op("binary " #op, &lhs_type,		\
 					    &rhs_type);				\
 										\
-	err = c_integer_promotions(&lhs->prog->tindex, &lhs_type);		\
+	err = c_integer_promotions(drgn_object_program(lhs), &lhs_type);	\
 	if (err)								\
 		return err;							\
-	err = c_integer_promotions(&lhs->prog->tindex, &rhs_type);		\
+	err = c_integer_promotions(drgn_object_program(lhs), &rhs_type);	\
 	if (err)								\
 		return err;							\
 										\
@@ -3442,7 +3464,7 @@ struct drgn_error *c_op_##op_name(struct drgn_object *res,		\
 	if (!drgn_type_is_##check(type.underlying_type))		\
 		return drgn_error_unary_op("unary " #op, &type);	\
 									\
-	err = c_integer_promotions(&obj->prog->tindex, &type);		\
+	err = c_integer_promotions(drgn_object_program(obj), &type);	\
 	if (err)							\
 		return err;						\
 									\
