@@ -2,6 +2,10 @@
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <dirent.h>
+#include <elf.h>
+#include <gelf.h>
+#include <libelf.h>
+#include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
@@ -10,14 +14,21 @@
 #include <unistd.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 
-#include "internal.h"
-#include "dwarf_index.h"
+#include "bitops.h"
+#include "debug_info.h"
+#include "drgn.h"
+#include "error.h"
+#include "hash_table.h"
 #include "helpers.h"
+#include "language.h"
 #include "linux_kernel.h"
+#include "memory_reader.h"
+#include "mread.h"
+#include "platform.h"
 #include "program.h"
-#include "read.h"
+#include "type.h"
+#include "util.h"
 
 struct drgn_error *read_memory_via_pgtable(void *buf, uint64_t address,
 					   size_t count, uint64_t offset,
@@ -286,18 +297,18 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 				}
 			}
 
-			err = drgn_type_index_find_primitive(&prog->tindex,
-							     DRGN_C_TYPE_UNSIGNED_LONG,
-							     &qualified_type.type);
+			err = drgn_program_find_primitive_type(prog,
+							       DRGN_C_TYPE_UNSIGNED_LONG,
+							       &qualified_type.type);
 			if (err)
 				return err;
 			return drgn_object_set_unsigned(ret, qualified_type,
 							prog->page_offset, 0);
 		} else if (name_len == strlen("PAGE_SHIFT") &&
 			   memcmp(name, "PAGE_SHIFT", name_len) == 0) {
-			err = drgn_type_index_find_primitive(&prog->tindex,
-							     DRGN_C_TYPE_INT,
-							     &qualified_type.type);
+			err = drgn_program_find_primitive_type(prog,
+							       DRGN_C_TYPE_INT,
+							       &qualified_type.type);
 			if (err)
 				return err;
 			return drgn_object_set_signed(ret, qualified_type,
@@ -305,9 +316,9 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 						      0);
 		} else if (name_len == strlen("PAGE_SIZE") &&
 			   memcmp(name, "PAGE_SIZE", name_len) == 0) {
-			err = drgn_type_index_find_primitive(&prog->tindex,
-							     DRGN_C_TYPE_UNSIGNED_LONG,
-							     &qualified_type.type);
+			err = drgn_program_find_primitive_type(prog,
+							       DRGN_C_TYPE_UNSIGNED_LONG,
+							       &qualified_type.type);
 			if (err)
 				return err;
 			return drgn_object_set_unsigned(ret, qualified_type,
@@ -315,9 +326,9 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 							0);
 		} else if (name_len == strlen("PAGE_MASK") &&
 			   memcmp(name, "PAGE_MASK", name_len) == 0) {
-			err = drgn_type_index_find_primitive(&prog->tindex,
-							     DRGN_C_TYPE_UNSIGNED_LONG,
-							     &qualified_type.type);
+			err = drgn_program_find_primitive_type(prog,
+							       DRGN_C_TYPE_UNSIGNED_LONG,
+							       &qualified_type.type);
 			if (err)
 				return err;
 			return drgn_object_set_unsigned(ret, qualified_type,
@@ -330,9 +341,9 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 			err = linux_kernel_get_thread_size(prog, &thread_size);
 			if (err)
 				return err;
-			err = drgn_type_index_find_primitive(&prog->tindex,
-							     DRGN_C_TYPE_UNSIGNED_LONG,
-							     &qualified_type.type);
+			err = drgn_program_find_primitive_type(prog,
+							       DRGN_C_TYPE_UNSIGNED_LONG,
+							       &qualified_type.type);
 			if (err)
 				return err;
 			return drgn_object_set_unsigned(ret, qualified_type,
@@ -341,16 +352,16 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 			   memcmp(name, "UTS_RELEASE", name_len) == 0) {
 			size_t len;
 
-			err = drgn_type_index_find_primitive(&prog->tindex,
-							     DRGN_C_TYPE_CHAR,
-							     &qualified_type.type);
+			err = drgn_program_find_primitive_type(prog,
+							       DRGN_C_TYPE_CHAR,
+							       &qualified_type.type);
 			if (err)
 				return err;
 			qualified_type.qualifiers = DRGN_QUALIFIER_CONST;
 			len = strlen(prog->vmcoreinfo.osrelease);
-			err = drgn_type_index_array_type(&prog->tindex, len + 1,
-							 qualified_type, NULL,
-							 &qualified_type.type);
+			err = drgn_array_type_create(prog, qualified_type,
+						     len + 1, &drgn_language_c,
+						     &qualified_type.type);
 			if (err)
 				return err;
 			qualified_type.qualifiers = 0;
@@ -385,14 +396,15 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 
 struct kernel_module_iterator {
 	char *name;
-	FILE *file;
-	char *notes;
-	size_t notes_len, notes_capacity;
+	/* /proc/modules file or NULL. */
+	FILE *modules_file;
 	union {
+		/* If using /proc/modules. */
 		struct {
 			size_t name_capacity;
 			uint64_t start, end;
 		};
+		/* If not using /proc/modules. */
 		struct {
 			struct drgn_qualified_type module_type;
 			struct drgn_object mod, node, tmp1, tmp2, tmp3;
@@ -403,8 +415,8 @@ struct kernel_module_iterator {
 
 static void kernel_module_iterator_deinit(struct kernel_module_iterator *it)
 {
-	if (it->file) {
-		fclose(it->file);
+	if (it->modules_file) {
+		fclose(it->modules_file);
 	} else {
 		drgn_object_deinit(&it->tmp3);
 		drgn_object_deinit(&it->tmp2);
@@ -412,28 +424,25 @@ static void kernel_module_iterator_deinit(struct kernel_module_iterator *it)
 		drgn_object_deinit(&it->node);
 		drgn_object_deinit(&it->mod);
 	}
-	free(it->notes);
 	free(it->name);
 }
 
 static struct drgn_error *
 kernel_module_iterator_init(struct kernel_module_iterator *it,
-			    struct drgn_program *prog)
+			    struct drgn_program *prog, bool use_proc_and_sys)
 {
 	struct drgn_error *err;
 
 	it->name = NULL;
-	it->notes = NULL;
-	it->notes_len = it->notes_capacity = 0;
-	if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
-		it->file = fopen("/proc/modules", "r");
-		if (!it->file) {
+	if (use_proc_and_sys) {
+		it->modules_file = fopen("/proc/modules", "r");
+		if (!it->modules_file) {
 			return drgn_error_create_os("fopen", errno,
 						    "/proc/modules");
 		}
 		it->name_capacity = 0;
 	} else {
-		it->file = NULL;
+		it->modules_file = NULL;
 
 		err = drgn_program_find_type(prog, "struct module", NULL,
 					     &it->module_type);
@@ -472,12 +481,9 @@ err:
 static struct drgn_error *
 kernel_module_iterator_next_live(struct kernel_module_iterator *it)
 {
-	ssize_t ret;
-	char *p;
-	size_t size;
 
 	errno = 0;
-	ret = getline(&it->name, &it->name_capacity, it->file);
+	ssize_t ret = getline(&it->name, &it->name_capacity, it->modules_file);
 	if (ret == -1) {
 		if (errno) {
 			return drgn_error_create_os("getline", errno,
@@ -486,9 +492,10 @@ kernel_module_iterator_next_live(struct kernel_module_iterator *it)
 			return &drgn_stop;
 		}
 	}
-	p = strchr(it->name, ' ');
-	if (!p || sscanf(p + 1, "%zu %*s %*s %*s %" SCNx64, &size,
-			 &it->start) != 2) {
+	char *p = strchr(it->name, ' ');
+	size_t size;
+	if (!p ||
+	    sscanf(p + 1, "%zu %*s %*s %*s %" SCNx64, &size, &it->start) != 2) {
 		return drgn_error_create(DRGN_ERROR_OTHER,
 					 "could not parse /proc/modules");
 	}
@@ -497,41 +504,6 @@ kernel_module_iterator_next_live(struct kernel_module_iterator *it)
 	return NULL;
 }
 
-static struct drgn_error *
-kernel_module_iterator_next_offline(struct kernel_module_iterator *it)
-{
-	struct drgn_error *err;
-	uint64_t addr;
-	char *name;
-
-	err = drgn_object_member_dereference(&it->node, &it->node, "next");
-	if (err)
-		return err;
-	err = drgn_object_read(&it->node, &it->node);
-	if (err)
-		return err;
-	err = drgn_object_read_unsigned(&it->node, &addr);
-	if (err)
-		return err;
-	if (addr == it->head)
-		return &drgn_stop;
-
-	err = drgn_object_container_of(&it->mod, &it->node, it->module_type,
-				       "list");
-	if (err)
-		return err;
-
-	err = drgn_object_member_dereference(&it->tmp1, &it->mod, "name");
-	if (err)
-		return err;
-	err = drgn_object_read_c_string(&it->tmp1, &name);
-	if (err)
-		return err;
-	free(it->name);
-	it->name = name;
-	return NULL;
-}
-
 /**
  * Get the the next loaded kernel module.
  *
@@ -546,23 +518,49 @@ kernel_module_iterator_next_offline(struct kernel_module_iterator *it)
 static struct drgn_error *
 kernel_module_iterator_next(struct kernel_module_iterator *it)
 {
-	if (it->file)
+	if (it->modules_file)
 		return kernel_module_iterator_next_live(it);
-	else
-		return kernel_module_iterator_next_offline(it);
+
+	struct drgn_error *err;
+
+	err = drgn_object_member_dereference(&it->node, &it->node, "next");
+	if (err)
+		return err;
+	err = drgn_object_read(&it->node, &it->node);
+	if (err)
+		return err;
+	uint64_t addr;
+	err = drgn_object_read_unsigned(&it->node, &addr);
+	if (err)
+		return err;
+	if (addr == it->head)
+		return &drgn_stop;
+
+	err = drgn_object_container_of(&it->mod, &it->node, it->module_type,
+				       "list");
+	if (err)
+		return err;
+
+	err = drgn_object_member_dereference(&it->tmp1, &it->mod, "name");
+	if (err)
+		return err;
+	char *name;
+	err = drgn_object_read_c_string(&it->tmp1, &name);
+	if (err)
+		return err;
+	free(it->name);
+	it->name = name;
+	return NULL;
 }
 
 struct kernel_module_section_iterator {
 	struct kernel_module_iterator *kmod_it;
-	DIR *dir;
-	union {
-		int dirfd;
-		struct {
-			uint64_t i;
-			uint64_t nsections;
-			char *name;
-		};
-	};
+	/* /sys/module/$module/sections directory or NULL. */
+	DIR *sections_dir;
+	/* If not using /sys/module/$module/sections. */
+	uint64_t i;
+	uint64_t nsections;
+	char *name;
 };
 
 static struct drgn_error *
@@ -572,31 +570,21 @@ kernel_module_section_iterator_init(struct kernel_module_section_iterator *it,
 	struct drgn_error *err;
 
 	it->kmod_it = kmod_it;
-	if (kmod_it->file) {
+	if (kmod_it->modules_file) {
 		char *path;
-
 		if (asprintf(&path, "/sys/module/%s/sections",
 			     kmod_it->name) == -1)
 			return &drgn_enomem;
-		it->dir = opendir(path);
+		it->sections_dir = opendir(path);
 		free(path);
-		if (!it->dir) {
-			err = drgn_error_format_os("opendir", errno,
-						   "/sys/module/%s/sections",
-						   kmod_it->name);
-			return err;
-		}
-		it->dirfd = dirfd(it->dir);
-		if (it->dirfd == -1) {
-			err = drgn_error_format_os("dirfd", errno,
-						   "/sys/module/%s/sections",
-						   kmod_it->name);
-			closedir(it->dir);
-			return err;
+		if (!it->sections_dir) {
+			return drgn_error_format_os("opendir", errno,
+						    "/sys/module/%s/sections",
+						    kmod_it->name);
 		}
 		return NULL;
 	} else {
-		it->dir = NULL;
+		it->sections_dir = NULL;
 		it->i = 0;
 		it->name = NULL;
 		/* it->nsections = mod->sect_attrs->nsections */
@@ -623,8 +611,8 @@ kernel_module_section_iterator_init(struct kernel_module_section_iterator *it,
 static void
 kernel_module_section_iterator_deinit(struct kernel_module_section_iterator *it)
 {
-	if (it->dir)
-		closedir(it->dir);
+	if (it->sections_dir)
+		closedir(it->sections_dir);
 	else
 		free(it->name);
 }
@@ -635,18 +623,14 @@ kernel_module_section_iterator_next_live(struct kernel_module_section_iterator *
 					 uint64_t *address_ret)
 {
 	struct dirent *ent;
-
-	while ((errno = 0, ent = readdir(it->dir))) {
-		int fd;
-		FILE *file;
-		int ret;
-
+	while ((errno = 0, ent = readdir(it->sections_dir))) {
 		if (ent->d_type == DT_DIR)
 			continue;
 		if (ent->d_type == DT_UNKNOWN) {
 			struct stat st;
 
-			if (fstatat(it->dirfd, ent->d_name, &st, 0) == -1) {
+			if (fstatat(dirfd(it->sections_dir), ent->d_name, &st,
+				    0) == -1) {
 				return drgn_error_format_os("fstatat", errno,
 							    "/sys/module/%s/sections/%s",
 							    it->kmod_it->name,
@@ -656,19 +640,19 @@ kernel_module_section_iterator_next_live(struct kernel_module_section_iterator *
 				continue;
 		}
 
-		fd = openat(it->dirfd, ent->d_name, O_RDONLY);
+		int fd = openat(dirfd(it->sections_dir), ent->d_name, O_RDONLY);
 		if (fd == -1) {
 			return drgn_error_format_os("openat", errno,
 						    "/sys/module/%s/sections/%s",
 						    it->kmod_it->name,
 						    ent->d_name);
 		}
-		file = fdopen(fd, "r");
+		FILE *file = fdopen(fd, "r");
 		if (!file) {
 			close(fd);
 			return drgn_error_create_os("fdopen", errno, NULL);
 		}
-		ret = fscanf(file, "%" SCNx64, address_ret);
+		int ret = fscanf(file, "%" SCNx64, address_ret);
 		fclose(file);
 		if (ret != 1) {
 			return drgn_error_format(DRGN_ERROR_OTHER,
@@ -689,13 +673,17 @@ kernel_module_section_iterator_next_live(struct kernel_module_section_iterator *
 }
 
 static struct drgn_error *
-kernel_module_section_iterator_next_offline(struct kernel_module_section_iterator *it,
-					    const char **name_ret,
-					    uint64_t *address_ret)
+kernel_module_section_iterator_next(struct kernel_module_section_iterator *it,
+				    const char **name_ret,
+				    uint64_t *address_ret)
 {
+	if (it->sections_dir) {
+		return kernel_module_section_iterator_next_live(it, name_ret,
+								address_ret);
+	}
+
 	struct drgn_error *err;
 	struct kernel_module_iterator *kmod_it = it->kmod_it;
-	char *name;
 
 	if (it->i >= it->nsections)
 		return &drgn_stop;
@@ -708,9 +696,27 @@ kernel_module_section_iterator_next_offline(struct kernel_module_section_iterato
 	err = drgn_object_read_unsigned(&kmod_it->tmp3, address_ret);
 	if (err)
 		return err;
+	/*
+	 * Since Linux kernel commit ed66f991bb19 ("module: Refactor section
+	 * attr into bin attribute") (in v5.8), the section name is
+	 * module_sect_attr.battr.attr.name. Before that, it is simply
+	 * module_sect_attr.name.
+	 */
+	err = drgn_object_member(&kmod_it->tmp2, &kmod_it->tmp2, "battr");
+	if (!err) {
+		err = drgn_object_member(&kmod_it->tmp2, &kmod_it->tmp2,
+					 "attr");
+		if (err)
+			return err;
+	} else {
+		if (err->code != DRGN_ERROR_LOOKUP)
+			return err;
+		drgn_error_destroy(err);
+	}
 	err = drgn_object_member(&kmod_it->tmp3, &kmod_it->tmp2, "name");
 	if (err)
 		return err;
+	char *name;
 	err = drgn_object_read_c_string(&kmod_it->tmp3, &name);
 	if (err)
 		return err;
@@ -719,20 +725,6 @@ kernel_module_section_iterator_next_offline(struct kernel_module_section_iterato
 	return NULL;
 }
 
-static struct drgn_error *
-kernel_module_section_iterator_next(struct kernel_module_section_iterator *it,
-				    const char **name_ret,
-				    uint64_t *address_ret)
-{
-	if (it->dir) {
-		return kernel_module_section_iterator_next_live(it, name_ret,
-								address_ret);
-	} else {
-		return kernel_module_section_iterator_next_offline(it, name_ret,
-								   address_ret);
-	}
-}
-
 /*
  * /lib/modules/$(uname -r)/modules.dep.bin maps all installed kernel modules to
  * their filesystem path (and dependencies, which we don't care about). It is
@@ -753,12 +745,10 @@ struct kmod_index {
 static struct drgn_error *kmod_index_validate(struct kmod_index *index,
 					      const char *path)
 {
-	const char *ptr;
+	const char *ptr = index->ptr;
 	uint32_t magic, version;
-
-	ptr = index->ptr;
-	if (!read_be32(&ptr, index->end, &magic) ||
-	    !read_be32(&ptr, index->end, &version)) {
+	if (!mread_be32(&ptr, index->end, &magic) ||
+	    !mread_be32(&ptr, index->end, &version)) {
 		return drgn_error_format(DRGN_ERROR_OTHER, "%s is too short",
 					 path);
 	}
@@ -824,20 +814,21 @@ static const char *kmod_index_find(struct kmod_index *index, const char *key)
 	static const uint32_t INDEX_NODE_CHILDS = UINT32_C(0x20000000);
 	static const uint32_t INDEX_NODE_VALUES = UINT32_C(0x40000000);
 	static const uint32_t INDEX_NODE_PREFIX = UINT32_C(0x80000000);
+
+	/* kmod_index_validate() already checked that this is within bounds. */
 	const char *ptr = index->ptr + 8;
 	uint32_t offset;
-
 	for (;;) {
-		if (!read_be32(&ptr, index->end, &offset))
+		if (!mread_be32(&ptr, index->end, &offset) ||
+		    !(ptr = mread_begin(index->ptr, index->end,
+					offset & INDEX_NODE_MASK)))
 			return NULL;
-		ptr = index->ptr + (offset & INDEX_NODE_MASK);
 
 		if (offset & INDEX_NODE_PREFIX) {
 			const char *prefix;
 			size_t prefix_len;
-
-			if (!read_string(&ptr, index->end, &prefix,
-					 &prefix_len))
+			if (!mread_string(&ptr, index->end, &prefix,
+					  &prefix_len))
 				return NULL;
 			if (strncmp(key, prefix, prefix_len) != 0)
 				return NULL;
@@ -846,20 +837,21 @@ static const char *kmod_index_find(struct kmod_index *index, const char *key)
 
 		if (offset & INDEX_NODE_CHILDS) {
 			uint8_t first, last;
-
-			if (!read_u8(&ptr, index->end, &first) ||
-			    !read_u8(&ptr, index->end, &last))
+			if (!mread_u8(&ptr, index->end, &first) ||
+			    !mread_u8(&ptr, index->end, &last))
 				return NULL;
 			if (*key) {
 				uint8_t cur = *key;
-
-				if (cur < first || cur > last)
+				if (cur < first || cur > last ||
+				    !mread_skip(&ptr, index->end,
+						4 * (cur - first)))
 					return NULL;
-				ptr += 4 * (cur - first);
 				key++;
 				continue;
 			} else {
-				ptr += 4 * (last - first + 1);
+				if (!mread_skip(&ptr, index->end,
+						4 * (last - first + 1)))
+					return NULL;
 				break;
 			}
 		} else if (*key) {
@@ -904,27 +896,24 @@ static void depmod_index_deinit(struct depmod_index *depmod)
 static bool depmod_index_find(struct depmod_index *depmod, const char *name,
 			      const char **path_ret, size_t *len_ret)
 {
-	const char *ptr;
-	uint32_t value_count;
-	const char *deps;
-	size_t deps_len;
-	char *colon;
-
-	ptr = kmod_index_find(&depmod->modules_dep, name);
+	const char *ptr = kmod_index_find(&depmod->modules_dep, name);
 	if (!ptr)
 		return false;
 
-	if (!read_be32(&ptr, depmod->modules_dep.end, &value_count) ||
+	uint32_t value_count;
+	if (!mread_be32(&ptr, depmod->modules_dep.end, &value_count) ||
 	    !value_count)
 		return false;
 
 	/* Skip over priority. */
-	ptr += 4;
-	if (!read_string(&ptr, depmod->modules_dep.end, &deps,
+	const char *deps;
+	size_t deps_len;
+	if (!mread_skip(&ptr, depmod->modules_dep.end, 4) ||
+	    !mread_string(&ptr, depmod->modules_dep.end, &deps,
 			 &deps_len))
 		return false;
 
-	colon = strchr(deps, ':');
+	const char *colon = strchr(deps, ':');
 	if (!colon)
 		return false;
 
@@ -1038,8 +1027,8 @@ get_kernel_module_name_from_this_module(Elf_Scn *this_module_scn,
 	return NULL;
 }
 
-DEFINE_HASH_MAP(elf_scn_name_map, const char *, Elf_Scn *, c_string_hash,
-		c_string_eq)
+DEFINE_HASH_MAP(elf_scn_name_map, const char *, Elf_Scn *,
+		c_string_key_hash_pair, c_string_key_eq)
 
 static struct drgn_error *
 cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
@@ -1155,57 +1144,53 @@ kernel_module_table_key(struct kernel_module_file * const *entry)
 }
 
 DEFINE_HASH_TABLE(kernel_module_table, struct kernel_module_file *,
-		  kernel_module_table_key, c_string_hash, c_string_eq)
+		  kernel_module_table_key, c_string_key_hash_pair,
+		  c_string_key_eq)
 
 static struct drgn_error *
-report_loaded_kernel_module(struct drgn_program *prog,
-			    struct drgn_dwarf_index *dindex,
+report_loaded_kernel_module(struct drgn_debug_info_load_state *load,
 			    struct kernel_module_iterator *kmod_it,
 			    struct kernel_module_table *kmod_table)
 {
 	struct drgn_error *err;
-	const char *name = kmod_it->name;
-	struct hash_pair hp;
-	struct kernel_module_table_iterator it;
-	struct kernel_module_file *kmod;
 
-	hp = kernel_module_table_hash(&name);
-	it = kernel_module_table_search_hashed(kmod_table, &name, hp);
+	const char *name = kmod_it->name;
+	struct hash_pair hp = kernel_module_table_hash(&name);
+	struct kernel_module_table_iterator it =
+		kernel_module_table_search_hashed(kmod_table, &name, hp);
 	if (!it.entry)
 		return &drgn_not_found;
 
-	kmod = *it.entry;
+	struct kernel_module_file *kmod = *it.entry;
 	kernel_module_table_delete_iterator_hashed(kmod_table, it, hp);
 	do {
 		uint64_t start, end;
-
 		err = cache_kernel_module_sections(kmod_it, kmod->elf, &start,
 						   &end);
 		if (err) {
-			err = drgn_dwarf_index_report_error(dindex,
-							    kmod->path,
-							    "could not get section addresses",
-							    err);
+			err = drgn_debug_info_report_error(load, kmod->path,
+							   "could not get section addresses",
+							   err);
 			if (err)
 				return err;
-			continue;
+			goto next;
 		}
 
-		err = drgn_dwarf_index_report_elf(dindex, kmod->path, kmod->fd,
-						  kmod->elf, start, end,
-						  kmod->name, NULL);
+		err = drgn_debug_info_report_elf(load, kmod->path, kmod->fd,
+						 kmod->elf, start, end,
+						 kmod->name, NULL);
 		kmod->elf = NULL;
 		kmod->fd = -1;
 		if (err)
 			return err;
+next:
 		kmod = kmod->next;
 	} while (kmod);
 	return NULL;
 }
 
 static struct drgn_error *
-report_default_kernel_module(struct drgn_program *prog,
-			     struct drgn_dwarf_index *dindex,
+report_default_kernel_module(struct drgn_debug_info_load_state *load,
 			     struct kernel_module_iterator *kmod_it,
 			     struct depmod_index *depmod)
 {
@@ -1216,71 +1201,73 @@ report_default_kernel_module(struct drgn_program *prog,
 		NULL,
 	};
 	struct drgn_error *err;
+
 	const char *depmod_path;
 	size_t depmod_path_len;
-	size_t extension_len;
-	char *path;
-	int fd;
-	Elf *elf;
-	uint64_t start, end;
-
 	if (!depmod_index_find(depmod, kmod_it->name, &depmod_path,
 			       &depmod_path_len)) {
-		return drgn_dwarf_index_report_error(dindex, kmod_it->name,
-						     "could not find module in depmod",
-						     NULL);
+		return drgn_debug_info_report_error(load, kmod_it->name,
+						    "could not find module in depmod",
+						    NULL);
 	}
 
+	size_t extension_len;
 	if (depmod_path_len >= 3 &&
 	    (memcmp(depmod_path + depmod_path_len - 3, ".gz", 3) == 0 ||
 	     memcmp(depmod_path + depmod_path_len - 3, ".xz", 3) == 0))
 		extension_len = 3;
 	else
 		extension_len = 0;
+	char *path;
+	int fd;
+	Elf *elf;
 	err = find_elf_file(&path, &fd, &elf, module_paths,
-			    prog->vmcoreinfo.osrelease,
+			    load->dbinfo->prog->vmcoreinfo.osrelease,
 			    depmod_path_len - extension_len, depmod_path,
 			    extension_len,
 			    depmod_path + depmod_path_len - extension_len);
 	if (err)
-		return drgn_dwarf_index_report_error(dindex, NULL, NULL, err);
+		return drgn_debug_info_report_error(load, NULL, NULL, err);
 	if (!elf) {
-		return drgn_dwarf_index_report_error(dindex, kmod_it->name,
-						     "could not find .ko",
-						     NULL);
+		return drgn_debug_info_report_error(load, kmod_it->name,
+						    "could not find .ko",
+						    NULL);
 	}
 
+	uint64_t start, end;
 	err = cache_kernel_module_sections(kmod_it, elf, &start, &end);
 	if (err) {
+		err = drgn_debug_info_report_error(load, path,
+						   "could not get section addresses",
+						   err);
 		elf_end(elf);
 		close(fd);
 		free(path);
-		return drgn_dwarf_index_report_error(dindex, path,
-						     "could not get section addresses",
-						     err);
+		return err;
 	}
 
-	err = drgn_dwarf_index_report_elf(dindex, path, fd, elf, start, end,
-					  kmod_it->name, NULL);
+	err = drgn_debug_info_report_elf(load, path, fd, elf, start, end,
+					 kmod_it->name, NULL);
 	free(path);
 	return err;
 }
 
 static struct drgn_error *
-report_loaded_kernel_modules(struct drgn_program *prog,
-			     struct drgn_dwarf_index *dindex,
+report_loaded_kernel_modules(struct drgn_debug_info_load_state *load,
 			     struct kernel_module_table *kmod_table,
-			     struct depmod_index *depmod)
+			     struct depmod_index *depmod,
+			     bool use_proc_and_sys)
 {
+	struct drgn_program *prog = load->dbinfo->prog;
 	struct drgn_error *err;
-	struct kernel_module_iterator kmod_it;
 
-	err = kernel_module_iterator_init(&kmod_it, prog);
+	struct kernel_module_iterator kmod_it;
+	err = kernel_module_iterator_init(&kmod_it, prog, use_proc_and_sys);
 	if (err) {
 kernel_module_iterator_error:
-		return drgn_dwarf_index_report_error(dindex, "kernel modules",
-						     "could not find loaded kernel modules",
-						     err);
+		return drgn_debug_info_report_error(load, "kernel modules",
+						    "could not find loaded kernel modules",
+						    err);
 	}
 	for (;;) {
 		err = kernel_module_iterator_next(&kmod_it);
@@ -1294,8 +1281,8 @@ kernel_module_iterator_error:
 
 		/* Look for an explicitly-reported file first. */
 		if (kmod_table) {
-			err = report_loaded_kernel_module(prog, dindex,
-							  &kmod_it, kmod_table);
+			err = report_loaded_kernel_module(load, &kmod_it,
+							  kmod_table);
 			if (!err)
 				continue;
 			else if (err != &drgn_not_found)
@@ -1308,24 +1295,24 @@ kernel_module_iterator_error:
 		 * already indexed that module.
 		 */
 		if (depmod &&
-		    !drgn_dwarf_index_is_indexed(dindex, kmod_it.name)) {
+		    !drgn_debug_info_is_indexed(load->dbinfo, kmod_it.name)) {
 			if (!depmod->modules_dep.ptr) {
 				err = depmod_index_init(depmod,
 							prog->vmcoreinfo.osrelease);
 				if (err) {
 					depmod->modules_dep.ptr = NULL;
-					err = drgn_dwarf_index_report_error(dindex,
-									    "kernel modules",
-									    "could not read depmod",
-									    err);
+					err = drgn_debug_info_report_error(load,
+									   "kernel modules",
+									   "could not read depmod",
+									   err);
 					if (err)
 						break;
 					depmod = NULL;
 					continue;
 				}
 			}
-			err = report_default_kernel_module(prog, dindex,
-							   &kmod_it, depmod);
+			err = report_default_kernel_module(load, &kmod_it,
+							   depmod);
 			if (err)
 				break;
 		}
@@ -1335,27 +1322,36 @@ kernel_module_iterator_error:
 }
 
 static struct drgn_error *
-report_kernel_modules(struct drgn_program *prog,
-		      struct drgn_dwarf_index *dindex,
+report_kernel_modules(struct drgn_debug_info_load_state *load,
 		      struct kernel_module_file *kmods, size_t num_kmods,
-		      bool report_default, bool need_module_definition,
-		      bool vmlinux_is_pending)
+		      bool need_module_definition, bool vmlinux_is_pending)
 {
+	struct drgn_program *prog = load->dbinfo->prog;
 	struct drgn_error *err;
 
-	if (!num_kmods && !report_default)
+	if (!num_kmods && !load->load_default)
 		return NULL;
 
 	/*
-	 * If we're not debugging the running kernel, then we need to index
-	 * vmlinux now so that we can walk the list of modules in the kernel.
+	 * If we're debugging the running kernel, we can get the loaded kernel
+	 * modules from /proc and /sys instead of from the core dump. This fast
+	 * path can be disabled via an environment variable for testing.
+	 */
+	bool use_proc_and_sys = false;
+	if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
+		char *env = getenv("DRGN_USE_PROC_AND_SYS_MODULES");
+		use_proc_and_sys = !env || atoi(env);
+	}
+	/*
+	 * If we're not using /proc and /sys, then we need to index vmlinux now
+	 * so that we can walk the list of modules in the kernel.
 	 *
 	 * If we need the definition of struct module to get the name of any
 	 * kernel modules, then we also need to index vmlinux now.
 	 */
 	if (vmlinux_is_pending &&
-	    (!(prog->flags & DRGN_PROGRAM_IS_LIVE) || need_module_definition)) {
-		err = drgn_dwarf_index_flush(dindex, false);
+	    (!use_proc_and_sys || need_module_definition)) {
+		err = drgn_debug_info_report_flush(load);
 		if (err)
 			return err;
 	}
@@ -1371,10 +1367,10 @@ report_kernel_modules(struct drgn_program *prog,
 						       "name", &name_member);
 		}
 		if (err) {
-			return drgn_dwarf_index_report_error(dindex,
-							     "kernel modules",
-							     "could not get kernel module names",
-							     err);
+			return drgn_debug_info_report_error(load,
+							    "kernel modules",
+							    "could not get kernel module names",
+							    err);
 		}
 		module_name_offset = name_member.bit_offset / 8;
 	}
@@ -1390,18 +1386,18 @@ report_kernel_modules(struct drgn_program *prog,
 								      module_name_offset,
 								      &kmod->name);
 			if (err) {
-				err = drgn_dwarf_index_report_error(dindex,
-								    kmod->path,
-								    NULL, err);
+				err = drgn_debug_info_report_error(load,
+								   kmod->path,
+								   NULL, err);
 				if (err)
 					goto out;
 				continue;
 			}
 			if (!kmod->name) {
-				err = drgn_dwarf_index_report_error(dindex,
-								    kmod->path,
-								    "could not find kernel module name",
-								    NULL);
+				err = drgn_debug_info_report_error(load,
+								   kmod->path,
+								   "could not find kernel module name",
+								   NULL);
 				if (err)
 					goto out;
 				continue;
@@ -1425,9 +1421,10 @@ report_kernel_modules(struct drgn_program *prog,
 		}
 	}
 
-	err = report_loaded_kernel_modules(prog, dindex,
+	err = report_loaded_kernel_modules(load,
 					   num_kmods ? &kmod_table : NULL,
-					   report_default ? &depmod : NULL);
+					   load->load_default ? &depmod : NULL,
+					   use_proc_and_sys);
 	if (err)
 		goto out;
 
@@ -1436,10 +1433,9 @@ report_kernel_modules(struct drgn_program *prog,
 		struct kernel_module_file *kmod = *it.entry;
 		it = kernel_module_table_delete_iterator(&kmod_table, it);
 		do {
-			err = drgn_dwarf_index_report_elf(dindex, kmod->path,
-							  kmod->fd, kmod->elf,
-							  0, 0, kmod->name,
-							  NULL);
+			err = drgn_debug_info_report_elf(load, kmod->path,
+							 kmod->fd, kmod->elf, 0,
+							 0, kmod->name, NULL);
 			kmod->elf = NULL;
 			kmod->fd = -1;
 			if (err)
@@ -1455,9 +1451,9 @@ out:
 	return err;
 }
 
-static struct drgn_error *report_vmlinux(struct drgn_program *prog,
-					 struct drgn_dwarf_index *dindex,
-					 bool *vmlinux_is_pending)
+static struct drgn_error *
+report_vmlinux(struct drgn_debug_info_load_state *load,
+	       bool *vmlinux_is_pending)
 {
 	static const char * const vmlinux_paths[] = {
 		/*
@@ -1471,54 +1467,49 @@ static struct drgn_error *report_vmlinux(struct drgn_program *prog,
 		"/lib/modules/%s/vmlinux",
 		NULL,
 	};
+	struct drgn_program *prog = load->dbinfo->prog;
 	struct drgn_error *err;
+
 	char *path;
 	int fd;
 	Elf *elf;
-	uint64_t start, end;
-
 	err = find_elf_file(&path, &fd, &elf, vmlinux_paths,
 			    prog->vmcoreinfo.osrelease);
 	if (err)
-		return drgn_dwarf_index_report_error(dindex, NULL, NULL, err);
+		return drgn_debug_info_report_error(load, NULL, NULL, err);
 	if (!elf) {
 		err = drgn_error_format(DRGN_ERROR_OTHER,
 					"could not find vmlinux for %s",
 					prog->vmcoreinfo.osrelease);
-		return drgn_dwarf_index_report_error(dindex, "kernel", NULL,
-						     err);
+		return drgn_debug_info_report_error(load, "kernel", NULL, err);
 	}
 
+	uint64_t start, end;
 	err = elf_address_range(elf, prog->vmcoreinfo.kaslr_offset, &start,
 				&end);
 	if (err) {
-		err = drgn_dwarf_index_report_error(dindex, path, NULL, err);
+		err = drgn_debug_info_report_error(load, path, NULL, err);
 		elf_end(elf);
 		close(fd);
 		free(path);
 		return err;
 	}
 
-	err = drgn_dwarf_index_report_elf(dindex, path, fd, elf, start, end,
-					  "kernel", vmlinux_is_pending);
+	err = drgn_debug_info_report_elf(load, path, fd, elf, start, end,
+					 "kernel", vmlinux_is_pending);
 	free(path);
 	return err;
 }
 
 struct drgn_error *
-linux_kernel_report_debug_info(struct drgn_program *prog,
-			       struct drgn_dwarf_index *dindex,
-			       const char **paths, size_t n,
-			       bool report_default, bool report_main)
+linux_kernel_report_debug_info(struct drgn_debug_info_load_state *load)
 {
+	struct drgn_program *prog = load->dbinfo->prog;
 	struct drgn_error *err;
-	struct kernel_module_file *kmods;
-	size_t i, num_kmods = 0;
-	bool need_module_definition = false;
-	bool vmlinux_is_pending = false;
 
-	if (n) {
-		kmods = malloc_array(n, sizeof(*kmods));
+	struct kernel_module_file *kmods;
+	if (load->num_paths) {
+		kmods = malloc_array(load->num_paths, sizeof(*kmods));
 		if (!kmods)
 			return &drgn_enomem;
 	} else {
@@ -1530,27 +1521,29 @@ linux_kernel_report_debug_info(struct drgn_program *prog,
 	 * modules. So, this sets aside kernel modules and reports everything
 	 * else.
 	 */
-	for (i = 0; i < n; i++) {
-		const char *path = paths[i];
+	size_t num_kmods = 0;
+	bool need_module_definition = false;
+	bool vmlinux_is_pending = false;
+	for (size_t i = 0; i < load->num_paths; i++) {
+		const char *path = load->paths[i];
 		int fd;
 		Elf *elf;
-		Elf_Scn *this_module_scn, *modinfo_scn;
-		bool is_vmlinux;
-
 		err = open_elf_file(path, &fd, &elf);
 		if (err) {
-			err = drgn_dwarf_index_report_error(dindex, path, NULL,
-							    err);
+			err = drgn_debug_info_report_error(load, path, NULL,
+							   err);
 			if (err)
 				goto out;
 			continue;
 		}
 
+		Elf_Scn *this_module_scn, *modinfo_scn;
+		bool is_vmlinux;
 		err = identify_kernel_elf(elf, &this_module_scn, &modinfo_scn,
 					  &is_vmlinux);
 		if (err) {
-			err = drgn_dwarf_index_report_error(dindex, path, NULL,
-							    err);
+			err = drgn_debug_info_report_error(load, path, NULL,
+							   err);
 			elf_end(elf);
 			close(fd);
 			if (err)
@@ -1559,16 +1552,14 @@ linux_kernel_report_debug_info(struct drgn_program *prog,
 		}
 		if (this_module_scn || modinfo_scn) {
 			struct kernel_module_file *kmod = &kmods[num_kmods++];
-
 			kmod->path = path;
 			kmod->fd = fd;
 			kmod->elf = elf;
 			err = get_kernel_module_name_from_modinfo(modinfo_scn,
 								  &kmod->name);
 			if (err) {
-				err = drgn_dwarf_index_report_error(dindex,
-								    path, NULL,
-								    err);
+				err = drgn_debug_info_report_error(load, path,
+								   NULL, err);
 				if (err)
 					goto out;
 				continue;
@@ -1579,49 +1570,46 @@ linux_kernel_report_debug_info(struct drgn_program *prog,
 			}
 		} else if (is_vmlinux) {
 			uint64_t start, end;
-			bool is_new;
-
 			err = elf_address_range(elf,
 						prog->vmcoreinfo.kaslr_offset,
 						&start, &end);
 			if (err) {
 				elf_end(elf);
 				close(fd);
-				err = drgn_dwarf_index_report_error(dindex,
-								    path, NULL,
-								    err);
+				err = drgn_debug_info_report_error(load, path,
+								   NULL, err);
 				if (err)
 					goto out;
 				continue;
 			}
 
-			err = drgn_dwarf_index_report_elf(dindex, path, fd, elf,
-							  start, end, "kernel",
-							  &is_new);
+			bool is_new;
+			err = drgn_debug_info_report_elf(load, path, fd, elf,
+							 start, end, "kernel",
+							 &is_new);
 			if (err)
 				goto out;
 			if (is_new)
 				vmlinux_is_pending = true;
 		} else {
-			err = drgn_dwarf_index_report_elf(dindex, path, fd, elf,
-							  0, 0, NULL, NULL);
+			err = drgn_debug_info_report_elf(load, path, fd, elf, 0,
+							 0, NULL, NULL);
 			if (err)
 				goto out;
 		}
 	}
 
-	if (report_main && !vmlinux_is_pending &&
-	    !drgn_dwarf_index_is_indexed(dindex, "kernel")) {
-		err = report_vmlinux(prog, dindex, &vmlinux_is_pending);
+	if (load->load_main && !vmlinux_is_pending &&
+	    !drgn_debug_info_is_indexed(load->dbinfo, "kernel")) {
+		err = report_vmlinux(load, &vmlinux_is_pending);
 		if (err)
 			goto out;
 	}
 
-	err = report_kernel_modules(prog, dindex, kmods, num_kmods,
-				    report_default, need_module_definition,
-				    vmlinux_is_pending);
+	err = report_kernel_modules(load, kmods, num_kmods,
+				    need_module_definition, vmlinux_is_pending);
 out:
-	for (i = 0; i < num_kmods; i++) {
+	for (size_t i = 0; i < num_kmods; i++) {
 		elf_end(kmods[i].elf);
 		if (kmods[i].fd != -1)
 			close(kmods[i].fd);
