@@ -12,11 +12,8 @@
 #ifndef DRGN_MEMORY_READER_H
 #define DRGN_MEMORY_READER_H
 
-#include <stdbool.h>
-#include <stddef.h>
-#include <stdint.h>
-
 #include "binary_search_tree.h"
+#include "drgn.h"
 
 /**
  * @ingroup Internals
@@ -57,9 +54,9 @@ drgn_memory_segment_to_key(const struct drgn_memory_segment *entry)
 	return entry->address;
 }
 
-DEFINE_BINARY_SEARCH_TREE_TYPE(drgn_memory_segment_tree, struct
-			       drgn_memory_segment, node,
-			       drgn_memory_segment_to_key)
+DEFINE_BINARY_SEARCH_TREE_TYPE(drgn_memory_segment_tree,
+			       struct drgn_memory_segment,
+			       node, drgn_memory_segment_to_key)
 
 /**
  * Memory reader.
