@@ -2,10 +2,9 @@
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <string.h>
+#include <stdlib.h>
 
-#include "internal.h"
 #include "object_index.h"
-#include "type.h"
 
 void drgn_object_index_init(struct drgn_object_index *oindex)
 {
@@ -41,6 +40,13 @@ drgn_object_index_add_finder(struct drgn_object_index *oindex,
 	return NULL;
 }
 
+void drgn_object_index_remove_finder(struct drgn_object_index *oindex)
+{
+	struct drgn_object_finder *finder = oindex->finders->next;
+	free(oindex->finders);
+	oindex->finders = finder;
+}
+
 struct drgn_error *drgn_object_index_find(struct drgn_object_index *oindex,
 					  const char *name,
 					  const char *filename,
