@@ -1,10 +1,12 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
 // SPDX-License-Identifier: GPL-3.0+
 
+#include <stdbool.h>
+#include <stdlib.h>
 #include <string.h>
 
-#include "internal.h"
 #include "symbol.h"
+#include "util.h"
 
 LIBDRGN_PUBLIC void drgn_symbol_destroy(struct drgn_symbol *sym)
 {
