@@ -3,7 +3,8 @@
 
 import errno
 import os
-import os.path
+from pathlib import Path
+import re
 import shlex
 import shutil
 import socket
@@ -12,7 +13,6 @@ import tempfile
 
 from util import nproc, out_of_date
 
-
 # Script run as init in the virtual machine. This only depends on busybox. We
 # don't assume that any regular commands are built in (not even echo or test),
 # so we always explicitly run busybox.
@@ -21,20 +21,25 @@ _INIT_TEMPLATE = r"""#!{busybox} sh
 set -eu
 
 export BUSYBOX={busybox}
+
+trap '"$BUSYBOX" poweroff -f' EXIT
+
+umask 022
+
 HOSTNAME=vmtest
 VPORT_NAME=com.osandov.vmtest.0
+RELEASE=$("$BUSYBOX" uname -r)
+
 # Set up overlayfs on the temporary directory containing this script.
-mnt=$(dirname "$0")
-
-trap '$BUSYBOX poweroff -f' EXIT
-
+mnt=$("$BUSYBOX" dirname "$0")
 "$BUSYBOX" mount -t tmpfs tmpfs "$mnt"
-"$BUSYBOX" mkdir -m 755 "$mnt/upper" "$mnt/work" "$mnt/merged"
+"$BUSYBOX" mkdir "$mnt/upper" "$mnt/work" "$mnt/merged"
 
-"$BUSYBOX" mkdir -m 755 "$mnt/upper/dev" "$mnt/upper/etc" "$mnt/upper/mnt"
+"$BUSYBOX" mkdir "$mnt/upper/dev" "$mnt/upper/etc" "$mnt/upper/mnt"
 "$BUSYBOX" mkdir -m 555 "$mnt/upper/proc" "$mnt/upper/sys"
 "$BUSYBOX" mkdir -m 1777 "$mnt/upper/tmp"
 
+# Create configuration files.
 "$BUSYBOX" cat << EOF > "$mnt/upper/etc/hosts"
 127.0.0.1 localhost
 ::1 localhost
@@ -47,6 +52,7 @@ EOF
 cd /
 "$BUSYBOX" umount -l /mnt
 
+# Mount additional filesystems.
 "$BUSYBOX" mount -t devtmpfs -o nosuid,noexec dev /dev
 "$BUSYBOX" mount -t proc -o nosuid,nodev,noexec proc /proc
 "$BUSYBOX" mount -t sysfs -o nosuid,nodev,noexec sys /sys
@@ -56,9 +62,32 @@ cd /
 # which breaks some tests.
 "$BUSYBOX" mount -t tmpfs -o nosuid,nodev tmpfs /tmp
 
+# Load kernel modules.
+"$BUSYBOX" mkdir -p "/lib/modules/$RELEASE"
+"$BUSYBOX" mount -t 9p -o trans=virtio,cache=loose,ro modules "/lib/modules/$RELEASE"
+"$BUSYBOX" modprobe configs
+
+# Create static device nodes.
+"$BUSYBOX" grep -v '^#' "/lib/modules/$RELEASE/modules.devname" |
+while read -r module name node; do
+	name="/dev/$name"
+	dev=${{node#?}}
+	major=${{dev%%:*}}
+	minor=${{dev##*:}}
+	type=${{node%"${{dev}}"}}
+	"$BUSYBOX" mkdir -p "$("$BUSYBOX" dirname "$name")"
+	"$BUSYBOX" mknod "$name" "$type" "$major" "$minor"
+done
+"$BUSYBOX" ln -s /proc/self/fd /dev/fd
+"$BUSYBOX" ln -s /proc/self/fd/0 /dev/stdin
+"$BUSYBOX" ln -s /proc/self/fd/1 /dev/stdout
+"$BUSYBOX" ln -s /proc/self/fd/2 /dev/stderr
+
+# Configure networking.
 "$BUSYBOX" hostname "$HOSTNAME"
 "$BUSYBOX" ip link set lo up
 
+# Find virtio port.
 vport=
 for vport_dir in /sys/class/virtio-ports/*; do
 	if "$BUSYBOX" [ -r "$vport_dir/name" \
@@ -82,14 +111,6 @@ set -e
 """
 
 
-def install_vmlinux_precommand(command: str, vmlinux: str) -> str:
-    vmlinux = shlex.quote(os.path.abspath(vmlinux))
-    return fr""""$BUSYBOX" mkdir -m 755 -p /boot &&
-	release=$("$BUSYBOX" uname -r) &&
-	"$BUSYBOX" ln -sf {vmlinux} "/boot/vmlinux-$release" &&
-	{command}"""
-
-
 def _compile(
     *args: str,
     CPPFLAGS: str = "",
@@ -116,18 +137,16 @@ def _compile(
     subprocess.check_call(cmd)
 
 
-def _build_onoatimehack(dir: str) -> str:
-    os.makedirs(dir, exist_ok=True)
+def _build_onoatimehack(dir: Path) -> Path:
+    dir.mkdir(parents=True, exist_ok=True)
 
-    onoatimehack_so = os.path.join(dir, "onoatimehack.so")
-    onoatimehack_c = os.path.relpath(
-        os.path.join(os.path.dirname(__file__), "onoatimehack.c")
-    )
+    onoatimehack_so = dir / "onoatimehack.so"
+    onoatimehack_c = (Path(__file__).parent / "onoatimehack.c").relative_to(Path.cwd())
     if out_of_date(onoatimehack_so, onoatimehack_c):
         _compile(
             "-o",
-            onoatimehack_so,
-            onoatimehack_c,
+            str(onoatimehack_so),
+            str(onoatimehack_c),
             CPPFLAGS="-D_GNU_SOURCE",
             CFLAGS="-fPIC",
             LDFLAGS="-shared",
@@ -140,33 +159,38 @@ class LostVMError(Exception):
     pass
 
 
-def run_in_vm(command: str, *, vmlinuz: str, build_dir: str) -> int:
-    # multidevs was added in QEMU 4.2.0.
-    if (
-        "multidevs"
-        in subprocess.run(
-            ["qemu-system-x86_64", "-help"],
-            stdout=subprocess.PIPE,
-            universal_newlines=True,
-        ).stdout
-    ):
-        multidevs = ",multidevs=remap"
-    else:
-        multidevs = ""
+def run_in_vm(command: str, kernel_dir: Path, build_dir: Path) -> int:
+    match = re.search(
+        "QEMU emulator version ([0-9]+(?:\.[0-9]+)*)",
+        subprocess.check_output(
+            ["qemu-system-x86_64", "-version"], universal_newlines=True
+        ),
+    )
+    if not match:
+        raise Exception("could not determine QEMU version")
+    qemu_version = tuple(int(x) for x in match.group(1).split("."))
 
-    onoatimehack = _build_onoatimehack(build_dir)
+    # multidevs was added in QEMU 4.2.0.
+    multidevs = ",multidevs=remap" if qemu_version >= (4, 2) else ""
+    # QEMU's 9pfs O_NOATIME handling was fixed in 5.1.0. The fix was backported
+    # to 5.0.1.
+    env = os.environ.copy()
+    if qemu_version < (5, 0, 1):
+        onoatimehack_so = _build_onoatimehack(build_dir)
+        env["LD_PRELOAD"] = f"{str(onoatimehack_so)}:{env.get('LD_PRELOAD', '')}"
 
     with tempfile.TemporaryDirectory(prefix="drgn-vmtest-") as temp_dir, socket.socket(
         socket.AF_UNIX
     ) as server_sock:
-        socket_path = os.path.join(temp_dir, "socket")
-        server_sock.bind(socket_path)
+        temp_path = Path(temp_dir)
+        socket_path = temp_path / "socket"
+        server_sock.bind(str(socket_path))
         server_sock.listen()
 
         busybox = shutil.which("busybox")
         if busybox is None:
             raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), "busybox")
-        init = os.path.abspath(os.path.join(temp_dir, "init"))
+        init = (temp_path / "init").resolve()
         with open(init, "w") as init_file:
             init_file.write(
                 _INIT_TEMPLATE.format(
@@ -190,20 +214,20 @@ def run_in_vm(command: str, *, vmlinuz: str, build_dir: str) -> int:
                 "-virtfs",
                 f"local,id=root,path=/,mount_tag=/dev/root,security_model=none,readonly{multidevs}",
 
+                "-virtfs",
+                f"local,path={kernel_dir},mount_tag=modules,security_model=none,readonly",
+
                 "-device", "virtio-serial",
                 "-chardev", f"socket,id=vmtest,path={socket_path}",
                 "-device",
                 "virtserialport,chardev=vmtest,name=com.osandov.vmtest.0",
 
-                "-kernel", vmlinuz,
+                "-kernel", str(kernel_dir / "vmlinuz"),
                 "-append",
                 f"rootfstype=9p rootflags=trans=virtio,cache=loose ro console=0,115200 panic=-1 init={init}",
                 # fmt: on
             ],
-            env={
-                **os.environ,
-                "LD_PRELOAD": f"{onoatimehack}:{os.getenv('LD_PRELOAD', '')}",
-            },
+            env=env,
         ) as qemu:
             server_sock.settimeout(5)
             try:
@@ -235,8 +259,6 @@ if __name__ == "__main__":
     import argparse
     import sys
 
-    from vmtest.resolver import KernelResolver
-
     parser = argparse.ArgumentParser(
         description="run vmtest virtual machine",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
@@ -268,18 +290,20 @@ if __name__ == "__main__":
     )
     args = parser.parse_args()
 
-    with KernelResolver(
-        [getattr(args, "kernel", "*")], download_dir=args.directory
-    ) as resolver:
-        kernel = next(iter(resolver))
-        try:
-            command = " ".join(args.command) if args.command else '"$BUSYBOX" sh -i'
-            command = install_vmlinux_precommand(command, kernel.vmlinux)
-            sys.exit(
-                run_in_vm(
-                    command=command, vmlinuz=kernel.vmlinuz, build_dir=args.directory
-                )
-            )
-        except LostVMError as e:
-            print("error:", e, file=sys.stderr)
-            sys.exit(args.lost_status)
+    kernel = getattr(args, "kernel", "*")
+    if kernel.startswith(".") or kernel.startswith("/"):
+        kernel_dir = Path(kernel)
+    else:
+        from vmtest.download import KernelDownloader
+
+        with KernelDownloader(
+            [getattr(args, "kernel", "*")], download_dir=Path(args.directory)
+        ) as downloader:
+            kernel_dir = next(iter(downloader))
+
+    try:
+        command = " ".join(args.command) if args.command else '"$BUSYBOX" sh -i'
+        sys.exit(run_in_vm(command, kernel_dir, Path(args.directory)))
+    except LostVMError as e:
+        print("error:", e, file=sys.stderr)
+        sys.exit(args.lost_status)
